/**
 * CSE154 19sp Final Exam Problem 5 (PHP) Solution
 */
(function () {
  "use strict";
  const URL = "club154.php";
  window.addEventListener("load", init);

  /*
   * One possible solution is below:
   */
  function init() {
    fetch(URL + "?mode=all")
      .then(checkStatus)
      .then(populateChars)
      .catch(handleError);
  }

  function populateChars(response) {
    response = response.split("\n");
    for (let i = 0; i < response.length; i++) {
      if (response[i] !== "") {
        let charData = response[i];
        let img = document.createElement("img");
        img.src = "data/" + charData + "/avatar.png";
        img.alt = charData;
        img.id = charData;
        id("playground").appendChild(img);
        img.addEventListener("click", showChar);
      }
    }
  }

  function showChar() {
    let name = this.id;
    fetch(URL + "?mode=lookup&name=" + name)
      .then(checkStatus)
      .then(JSON.parse)
      .then(populateCharInfo)
      .catch(handleError);
    if (qs(".selected")) {
      qs(".selected").classList.remove("selected");
    }
    this.classList.add("selected");
  }

  function populateCharInfo(response) {
    id("name").innerText = response.name;
    id("series").innerText = response.series;
    id("description").innerText = response.description;
    id("appearances").innerHTML = "";
    for (let i = 0; i < response.appearances.length; i++) {
      let appearance = response.appearances[i];
      let li = document.createElement("li");
      li.innerText = appearance;
      id("appearances").appendChild(li);
    }
    id("spotlight").classList.remove("hidden");
  }

  function handleError(error) {
    id("error").classList.remove("hidden");
    id("spotlight").classList.add("hidden");
    id("error").innerText = error;
    if (qs(".selected")) {
      qs(".selected").classList.remove("selected");
    }
  }

  // Helper functions provided on exam

  /**
   * Returns the element that has the ID attribute with the specified value.
   * @param {string} idName - element ID
   * @returns {object} DOM object associated with id.
   */
  function id(idName) {
    return document.getElementById(idName);
  }

  /**
   * Returns the first element that matches the given CSS selector.
   * @param {string} selector - CSS query selector.
   * @returns {object} The first DOM object matching the query.
   */
  function qs(selector) {
    return document.querySelector(selector);
  }

  /**
   * Returns the array of elements that match the given CSS selector.
   * @param {string} selector - CSS query selector
   * @returns {object[]} array of DOM objects matching the query.
   */
  function qsa(selector) {
    return document.querySelectorAll(selector);
  }

  /**
   * Helper function to return the response's result text if successful, otherwise
   * returns the rejected Promise result with an error status and corresponding text
   * @param {object} response - response to check for success/error
   * @returns {object} - valid result text if response was successful, otherwise rejected
   *           Promise result
   */
  function checkStatus(response) {
  if (response.status >= 200 && response.status < 300 || response.status === 0) {
    return response.text();
  } else {
    return Promise.reject(new Error(response.status + ": " + response.statusText));
  }
  }
})();
